package com.jiuqi.util.crypto;

import com.jiuqi.util.JqLib;
import com.jiuqi.util.crypto.EncryptException;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.math.BigInteger;
import java.security.Key;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.RSAPrivateKeySpec;
import java.security.spec.RSAPublicKeySpec;
import javax.crypto.Cipher;
import org.bouncycastle.jce.provider.BouncyCastleProvider;

public class RSAUtil {

   public static KeyPair generateKeyPair() throws EncryptException {
      try {
         KeyPairGenerator e = KeyPairGenerator.getInstance("RSA", new BouncyCastleProvider());
         e.initialize(1024, new SecureRandom());
         KeyPair keyPair = e.genKeyPair();
         return keyPair;
      } catch (Exception var2) {
         throw new EncryptException(var2.getMessage());
      }
   }

   public static RSAPublicKey generateRSAPublicKey(byte[] modulus, byte[] publicExponent) throws EncryptException {
      KeyFactory keyFac = null;

      try {
         keyFac = KeyFactory.getInstance("RSA", new BouncyCastleProvider());
      } catch (NoSuchAlgorithmException var6) {
         throw new EncryptException(var6.getMessage());
      }

      RSAPublicKeySpec pubKeySpec = new RSAPublicKeySpec(new BigInteger(modulus), new BigInteger(publicExponent));

      try {
         return (RSAPublicKey)keyFac.generatePublic(pubKeySpec);
      } catch (InvalidKeySpecException var5) {
         throw new EncryptException(var5.getMessage());
      }
   }

   public static RSAPrivateKey generateRSAPrivateKey(byte[] modulus, byte[] privateExponent) throws EncryptException {
      KeyFactory keyFac = null;

      try {
         keyFac = KeyFactory.getInstance("RSA", new BouncyCastleProvider());
      } catch (NoSuchAlgorithmException var6) {
         throw new EncryptException(var6.getMessage());
      }

      RSAPrivateKeySpec priKeySpec = new RSAPrivateKeySpec(new BigInteger(modulus), new BigInteger(privateExponent));

      try {
         return (RSAPrivateKey)keyFac.generatePrivate(priKeySpec);
      } catch (InvalidKeySpecException var5) {
         throw new EncryptException(var5.getMessage());
      }
   }

   public static byte[] encrypt(Key key, byte[] data) throws EncryptException {
      try {
         Cipher e = Cipher.getInstance("RSA", new BouncyCastleProvider());
         e.init(1, key);
         int blockSize = e.getBlockSize();
         int outputSize = e.getOutputSize(data.length);
         int leavedSize = data.length % blockSize;
         int blocksSize = leavedSize != 0?data.length / blockSize + 1:data.length / blockSize;
         byte[] raw = new byte[outputSize * blocksSize];

         for(int i = 0; data.length - i * blockSize > 0; ++i) {
            if(data.length - i * blockSize > blockSize) {
               e.doFinal(data, i * blockSize, blockSize, raw, i * outputSize);
            } else {
               e.doFinal(data, i * blockSize, data.length - i * blockSize, raw, i * outputSize);
            }
         }

         return raw;
      } catch (Exception var9) {
         throw new EncryptException(var9.getMessage());
      }
   }

   public static byte[] decrypt(Key key, byte[] raw) throws EncryptException {
      try {
         Cipher e = Cipher.getInstance("RSA", new BouncyCastleProvider());
         e.init(2, key);
         int blockSize = e.getBlockSize();
         ByteArrayOutputStream bout = new ByteArrayOutputStream(64);

         for(int j = 0; raw.length - j * blockSize > 0; ++j) {
            bout.write(e.doFinal(raw, j * blockSize, blockSize));
         }

         return bout.toByteArray();
      } catch (Exception var6) {
         throw new EncryptException(var6.getMessage());
      }
   }

   public static void main(String[] args) throws Exception {
      File file = new File("d:\\test.html");
      FileInputStream in = new FileInputStream(file);
      ByteArrayOutputStream bout = new ByteArrayOutputStream();
      byte[] tmpbuf = new byte[1024];

      int count1;
      for(boolean count = false; (count1 = in.read(tmpbuf)) != -1; tmpbuf = new byte[1024]) {
         bout.write(tmpbuf, 0, count1);
      }

      in.close();
      byte[] orgData = bout.toByteArray();
      KeyPair keyPair = generateKeyPair();
      RSAPublicKey pubKey = (RSAPublicKey)keyPair.getPublic();
      RSAPrivateKey priKey = (RSAPrivateKey)keyPair.getPrivate();
      byte[] pubModBytes = pubKey.getModulus().toByteArray();
      byte[] pubPubExpBytes = pubKey.getPublicExponent().toByteArray();
      byte[] priModBytes = priKey.getModulus().toByteArray();
      byte[] priPriExpBytes = priKey.getPrivateExponent().toByteArray();
      RSAPublicKey recoveryPubKey = generateRSAPublicKey(pubModBytes, pubPubExpBytes);
      RSAPrivateKey recoveryPriKey = generateRSAPrivateKey(priModBytes, priPriExpBytes);
      System.out.println(JqLib.bytesToHexString(priModBytes).toString());
      System.out.println(JqLib.bytesToHexString(priPriExpBytes).toString());
      System.out.println(JqLib.bytesToHexString(pubModBytes).toString());
      System.out.println(JqLib.bytesToHexString(pubPubExpBytes).toString());
      byte[] raw = encrypt(recoveryPriKey, orgData);
      file = new File("d:\\encrypt_result.dat");
      FileOutputStream out = new FileOutputStream(file);
      out.write(raw);
      out.close();
      byte[] data = decrypt(recoveryPubKey, raw);
      file = new File("d:\\decrypt_result.html");
      out = new FileOutputStream(file);
      out.write(data);
      out.flush();
      out.close();
   }
}
